@@ -40,8 +40,6 @@ CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 THE POSSIBILITY OF SUCH DAMAGE.
 """
-from __future__ import division, print_function, absolute_import
-
 
 import numpy
 
@@ -87,13 +85,13 @@ def bytescale(data, cmin=None, cmax=None, high=255, low=0):
     ----------
     data : ndarray
         PIL image data array.
-    cmin : scalar, optional
+    cmin : scalar, default=None
         Bias scaling of small values. Default is ``data.min()``.
-    cmax : scalar, optional
+    cmax : scalar, default=None
         Bias scaling of large values. Default is ``data.max()``.
-    high : scalar, optional
+    high : scalar, default=None
         Scale max value to `high`.  Default is 255.
-    low : scalar, optional
+    low : scalar, default=None
         Scale min value to `low`.  Default is 0.
 
     Returns
@@ -158,9 +156,9 @@ def imread(name, flatten=False, mode=None):
     ----------
     name : str or file object
         The file name or file object to be read.
-    flatten : bool, optional
+    flatten : bool, default=False
         If True, flattens the color layers into a single gray-scale layer.
-    mode : str, optional
+    mode : str, default=None
         Mode to convert image to, e.g. ``'RGB'``.  See the Notes for more
         details.
 
@@ -229,7 +227,7 @@ def imsave(name, arr, format=None):
         represents a grey-level image.  Shape ``MxNx3`` stores the red, green
         and blue bands along the last dimension.  An alpha layer may be
         included, specified as the last colour band of an ``MxNx4`` array.
-    format : str
+    format : str, default=None
         Image format. If omitted, the format to use is determined from the
         file name extension. If a file object was used instead of a file name,
         this parameter should always be used.
@@ -272,9 +270,9 @@ def fromimage(im, flatten=False, mode=None):
     ----------
     im : PIL image
         Input image.
-    flatten : bool
+    flatten : bool, default=False
         If true, convert the output to grey-scale.
-    mode : str, optional
+    mode : str, default=None
         Mode to convert image to, e.g. ``'RGB'``.  See the Notes of the
         `imread` docstring for more details.
 
@@ -369,23 +367,23 @@ def toimage(arr, high=255, low=0, cmin=None, cmax=None, pal=None,
         shape = (shape[1], shape[0])  # columns show up first
         if mode == 'F':
             data32 = data.astype(numpy.float32)
-            image = Image.frombytes(mode, shape, data32.tostring())
+            image = Image.frombytes(mode, shape, data32.tobytes())
             return image
         if mode in [None, 'L', 'P']:
             bytedata = bytescale(data, high=high, low=low,
                                  cmin=cmin, cmax=cmax)
-            image = Image.frombytes('L', shape, bytedata.tostring())
+            image = Image.frombytes('L', shape, bytedata.tobytes())
             if pal is not None:
-                image.putpalette(asarray(pal, dtype=uint8).tostring())
+                image.putpalette(asarray(pal, dtype=uint8).tobytes())
                 # Becomes a mode='P' automagically.
             elif mode == 'P':  # default gray-scale
                 pal = (arange(0, 256, 1, dtype=uint8)[:, newaxis] *
                        ones((3,), dtype=uint8)[newaxis, :])
-                image.putpalette(asarray(pal, dtype=uint8).tostring())
+                image.putpalette(asarray(pal, dtype=uint8).tobytes())
             return image
         if mode == '1':  # high input gives threshold for 1
             bytedata = (data > high)
-            image = Image.frombytes('1', shape, bytedata.tostring())
+            image = Image.frombytes('1', shape, bytedata.tobytes())
             return image
         if cmin is None:
             cmin = amin(ravel(data))
@@ -394,7 +392,7 @@ def toimage(arr, high=255, low=0, cmin=None, cmax=None, pal=None,
         data = (data*1.0 - cmin)*(high - low)/(cmax - cmin) + low
         if mode == 'I':
             data32 = data.astype(numpy.uint32)
-            image = Image.frombytes(mode, shape, data32.tostring())
+            image = Image.frombytes(mode, shape, data32.tobytes())
         else:
             raise ValueError(_errstr)
         return image
@@ -419,13 +417,13 @@ def toimage(arr, high=255, low=0, cmin=None, cmax=None, pal=None,
 
     bytedata = bytescale(data, high=high, low=low, cmin=cmin, cmax=cmax)
     if ca == 2:
-        strdata = bytedata.tostring()
+        strdata = bytedata.tobytes()
         shape = (shape[1], shape[0])
     elif ca == 1:
-        strdata = transpose(bytedata, (0, 2, 1)).tostring()
+        strdata = transpose(bytedata, (0, 2, 1)).tobytes()
         shape = (shape[2], shape[0])
     elif ca == 0:
-        strdata = transpose(bytedata, (1, 2, 0)).tostring()
+        strdata = transpose(bytedata, (1, 2, 0)).tobytes()
         shape = (shape[2], shape[1])
     if mode is None:
         if numch == 3:
@@ -470,10 +468,10 @@ def imresize(arr, size, interp='bilinear', mode=None):
         * float - Fraction of current size.
         * tuple - Size of the output image (height, width).
 
-    interp : str, optional
+    interp : str, default='bilinear'
         Interpolation to use for re-sizing ('nearest', 'lanczos', 'bilinear',
         'bicubic' or 'cubic').
-    mode : str, optional
+    mode : str, default=None
         The PIL image mode ('P', 'L', etc.) to convert `arr` before resizing.
         If ``mode=None`` (the default), 2-D images will be treated like
         ``mode='L'``, i.e. casting to long integer.  For 3-D and 4-D arrays,
